#!/usr/bin/env python
# -*- coding: utf-8 -*-

from django.conf.urls import patterns, url  # pragma: no cover


urlpatterns = patterns(  # pragma: no cover
    "",
    url("^$", 'viewfunc', name='root-of-app'),
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import sys
import logging
import json
import uuid

from django.db import models
from django.conf import settings
from django.utils import timezone

from google.protobuf.descriptor import FieldDescriptor
from google.protobuf.any_pb2 import Any

logging.basicConfig()
LOGGER = logging.getLogger(__name__)
LOGGER.setLevel(logging.WARNING)
if settings.DEBUG:
    LOGGER.setLevel(logging.DEBUG)


PB_FIELD_TYPE_TIMESTAMP = FieldDescriptor.MAX_TYPE + 1
PB_FIELD_TYPE_REPEATED = FieldDescriptor.MAX_TYPE + 2
PB_FIELD_TYPE_MAP = FieldDescriptor.MAX_TYPE + 3
PB_FIELD_TYPE_MESSAGE = FieldDescriptor.MAX_TYPE + 4
PB_FIELD_TYPE_REPEATED_MESSAGE = FieldDescriptor.MAX_TYPE + 5
PB_FIELD_TYPE_MESSAGE_MAP = FieldDescriptor.MAX_TYPE + 6
PB_FIELD_TYPE_MESSAGE_ANY = FieldDescriptor.MAX_TYPE + 7


def _defaultfield_to_pb(pb_obj, pb_field, dj_field_value):
    """ handling any fields conversion to protobuf
    """
    LOGGER.debug("Django Value field, assign proto msg field: {} = {}".format(pb_field.name, dj_field_value))
    if sys.version_info < (3,) and type(dj_field_value) is buffer:
        dj_field_value = bytes(dj_field_value)
    setattr(pb_obj, pb_field.name, dj_field_value)


def _defaultfield_from_pb(instance, dj_field_name, pb_field, pb_value):
    """ handling any fields setting from protobuf
    """
    LOGGER.debug("Django Value Field, set dj field: {} = {}".format(dj_field_name, pb_value))
    setattr(instance, dj_field_name, pb_value)


def _datetimefield_to_pb(pb_obj, pb_field, dj_field_value):
    """handling Django DateTimeField field

    :param pb_obj: protobuf message obj which is return value of to_pb()
    :param pb_field: protobuf message field which is current processing field
    :param dj_field_value: Currently proecessing django field value
    :returns: None
    """
    if getattr(getattr(pb_obj, pb_field.name), 'FromDatetime', False):
        if settings.USE_TZ:
            dj_field_value = timezone.make_naive(dj_field_value, timezone=timezone.utc)
        getattr(pb_obj, pb_field.name).FromDatetime(dj_field_value)


def _datetimefield_from_pb(instance, dj_field_name, pb_field, pb_value):
    """handling datetime field (Timestamp) object to dj field

    :param dj_field_name: Currently target django field's name
    :param pb_value: Currently processing protobuf message value
    :returns: None
    """
    dt = pb_value.ToDatetime()
    if settings.USE_TZ:
        dt = timezone.localtime(timezone.make_aware(dt, timezone.utc))
    # FIXME: not datetime field
    setattr(instance, dj_field_name, dt)


def _uuid_to_pb(pb_obj, pb_field, dj_field_value):
    """handling Django UUIDField field

    :param pb_obj: protobuf message obj which is return value of to_pb()
    :param pb_field: protobuf message field which is current processing field
    :param dj_field_value: Currently proecessing django field value
    :returns: None
    """
    setattr(pb_obj, pb_field.name, str(dj_field_value))


def _uuid_from_pb(instance, dj_field_name, pb_field, pb_value):
    """handling string object to dj UUIDField

    :param dj_field_name: Currently target django field's name
    :param pb_value: Currently processing protobuf message value
    :returns: None
    """
    setattr(instance, dj_field_name, uuid.UUID(pb_value))


class ProtoBufFieldMixin(object):
    @staticmethod
    def to_pb(pb_obj, pb_field, dj_field_value):
        raise NotImplementedError()

    @staticmethod
    def from_pb(instance, dj_field_name, pb_field, pb_value):
        raise NotImplementedError()


class ProtoBufAnyField(models.BinaryField, ProtoBufFieldMixin):
    @staticmethod
    def to_pb(pb_obj, pb_field, dj_field_value):
        getattr(pb_obj, pb_field.name).CopyFrom(dj_field_value)

    @staticmethod
    def from_pb(instance, dj_field_name, pb_field, pb_value):
        setattr(instance, dj_field_name, pb_value)

    def pre_save(self, model_instance, add):
        value = getattr(model_instance, self.name)

        if not isinstance(value, Any):
            raise ValueError("Field it's not from type: google.protobuf.any_pb2.Any")

        return value.SerializeToString()

    def from_db_value(self, value, expression, connection, context=None):
        if value is None:
            return value

        _any = Any()
        _any.ParseFromString(value)
        return _any


class JSONField(models.TextField):
    def from_db_value(self, value, expression, connection, context=None):
        return self._deserialize(value)

    def to_python(self, value):
        if isinstance(value, str):
            return self._deserialize(value)

        return value

    def get_prep_value(self, value):
        return json.dumps(value)

    def _deserialize(self, value):
        if value is None:
            return None

        return json.loads(value)


class ArrayField(JSONField, ProtoBufFieldMixin):
    @staticmethod
    def to_pb(pb_obj, pb_field, dj_field_value):
        getattr(pb_obj, pb_field.name).extend(dj_field_value)

    @staticmethod
    def from_pb(instance, dj_field_name, pb_field, pb_value):
        setattr(instance, dj_field_name, list(pb_value))


class MapField(JSONField, ProtoBufFieldMixin):
    @staticmethod
    def to_pb(pb_obj, pb_field, dj_field_value):
        getattr(pb_obj, pb_field.name).update(dj_field_value)

    @staticmethod
    def from_pb(instance, dj_field_name, pb_field, pb_value):
        setattr(instance, dj_field_name, dict(pb_value))


class RepeatedMessageField(models.ManyToManyField, ProtoBufFieldMixin):
    class Descriptor(models.fields.related_descriptors.ManyToManyDescriptor):
        def __init__(self, field_name, index_field_name, rel, reverse=False):
            super(RepeatedMessageField.Descriptor, self).__init__(rel, reverse)
            self._field_name = field_name
            self._index_field_name = index_field_name

        def __get__(self, instance, cls=None):
            if instance is None:
                raise AttributeError('Can only be accessed via an instance.')

            if self._field_name not in instance.__dict__:
                instance.__dict__[self._field_name] = [self.related_manager_cls(instance).get(id=id_) for id_ in getattr(instance, self._index_field_name)]
            return instance.__dict__[self._field_name]

        def __set__(self, instance, value):
            instance.__dict__[self._field_name] = value

    def __init__(self, *args, **kwargs):
        super(RepeatedMessageField, self).__init__(default=[], *args, **kwargs)

    def deconstruct(self):
        name, path, args, kwargs = super(RepeatedMessageField, self).deconstruct()
        kwargs.pop('default')
        return name, path, args, kwargs

    def contribute_to_class(self, cls, name):
        index_field_name = '%s_index' % name
        index_field = JSONField(default=[], editable=False, blank=True)
        index_field.creation_counter = self.creation_counter
        cls.add_to_class(index_field_name, index_field)
        super(RepeatedMessageField, self).contribute_to_class(cls, name)
        setattr(cls, self.attname, RepeatedMessageField.Descriptor(name, index_field_name, self.remote_field, reverse=False))

    def save(self, instance):
        for message in getattr(instance, self.attname):
            type(instance).__dict__[self.attname].related_manager_cls(instance).add(message)
        setattr(instance, '%s_index' % self.attname, [q.id for q in instance.__dict__[self.attname]])

    def load(self, instance):
        getattr(instance, self.attname)

    @staticmethod
    def to_pb(pb_obj, pb_field, dj_field_value):
        getattr(pb_obj, pb_field.name).extend([m.to_pb() for m in dj_field_value])

    @staticmethod
    def from_pb(instance, dj_field_name, pb_field, pb_value):
        related_model = instance._meta.get_field(dj_field_name).related_model
        setattr(instance, dj_field_name, [related_model().from_pb(pb_message) for pb_message in pb_value])


class RepeatedForeignField(JSONField):
    """
    This is an naive proxy field for JSONField to support types cannot
    easily extend with protobuf support.
    """
    pass

class MessageMapField(models.ManyToManyField, ProtoBufFieldMixin):
    class Descriptor(models.fields.related_descriptors.ManyToManyDescriptor):
        def __init__(self, field_name, index_field_name, rel, reverse=False):
            super(MessageMapField.Descriptor, self).__init__(rel, reverse)
            self._field_name = field_name
            self._index_field_name = index_field_name

        def __get__(self, instance, cls=None):
            if instance is None:
                raise AttributeError('Can only be accessed via an instance.')

            if self._field_name not in instance.__dict__:
                instance.__dict__[self._field_name] = {key: self.related_manager_cls(instance).get(id=id_) for key, id_ in getattr(instance, self._index_field_name).items()}
            return instance.__dict__[self._field_name]

        def __set__(self, instance, value):
            instance.__dict__[self._field_name] = value

    def __init__(self, *args, **kwargs):
        super(MessageMapField, self).__init__(default={}, *args, **kwargs)

    def deconstruct(self):
        name, path, args, kwargs = super(MessageMapField, self).deconstruct()
        kwargs.pop('default')
        return name, path, args, kwargs

    def contribute_to_class(self, cls, name):
        index_field_name = '%s_index' % name
        index_field = JSONField(default={}, editable=False, blank=True)
        index_field.creation_counter = self.creation_counter
        cls.add_to_class(index_field_name, index_field)
        super(MessageMapField, self).contribute_to_class(cls, name)
        setattr(cls, self.attname, MessageMapField.Descriptor(name, index_field_name, self.remote_field, reverse=False))

    def save(self, instance):
        for message in getattr(instance, self.attname).values():
            type(instance).__dict__[self.attname].related_manager_cls(instance).add(message)
        setattr(instance, '%s_index' % self.attname, {key: message.id for key, message in instance.__dict__[self.attname].items()})

    def load(self, instance):
        getattr(instance, self.attname)

    @staticmethod
    def to_pb(pb_obj, pb_field, dj_field_value):
        for key in dj_field_value:
            getattr(pb_obj, pb_field.name)[key].CopyFrom(dj_field_value[key].to_pb())

    @staticmethod
    def from_pb(instance, dj_field_name, pb_field, pb_value):
        related_model = instance._meta.get_field(dj_field_name).related_model
        setattr(instance, dj_field_name, {key: related_model().from_pb(pb_message) for key, pb_message in pb_value.items()})



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging
import six

from django.db import models
from django.db.models.fields.reverse_related import ManyToOneRel
from django.conf import settings

from google.protobuf.descriptor import FieldDescriptor

from . import fields

logging.basicConfig()
LOGGER = logging.getLogger(__name__)
LOGGER.setLevel(logging.WARNING)
if settings.DEBUG:
    LOGGER.setLevel(logging.DEBUG)


class DjangoPBModelError(Exception):
    pass


class Meta(type(models.Model)):
    def __init__(self, name, bases, attrs):
        super(Meta, self).__init__(name, bases, attrs)
        self.pb_2_dj_field_serializers = self._pb_2_dj_default_field_serializers.copy()
        self.pb_2_dj_field_serializers.update(attrs.get('pb_2_dj_field_serializers', {}))

        self.pb_auto_field_type_mapping = self._pb_auto_field_type_mapping.copy()
        self.pb_auto_field_type_mapping.update(attrs.get('pb_auto_field_type_mapping', {}))

        if self._meta.proxy:
            # prevent duplicated field in proxy model
            # ref: https://github.com/myyang/django-pb-model/issues/29
            return

        if self.pb_model is not None:
            if self.pb_2_dj_fields == '__all__':
                self.pb_2_dj_fields = self.pb_model.DESCRIPTOR.fields_by_name.keys()

            for pb_field_name in self.pb_2_dj_fields:
                pb_field_descriptor = self.pb_model.DESCRIPTOR.fields_by_name[pb_field_name]
                dj_field_name = self.pb_2_dj_field_map.get(pb_field_name, pb_field_name)
                if not isinstance(dj_field_name, dict) and dj_field_name not in attrs:
                    field = self._create_field(pb_field_descriptor)
                    if field is not None:
                        field.contribute_to_class(self, dj_field_name)

    def _create_field(self, message_field):
        message_field_type = message_field.type

        if Meta._is_message_map_field(message_field):
            mapped_message = message_field.message_type.fields_by_name['value'].message_type
            return self._create_message_map_field(message_field.containing_type.name, mapped_message.name,
                                                  message_field.name)
        elif Meta._is_map_field(message_field):
            return self._create_map_field()
        elif Meta._is_repeated_message_field(message_field):
            return self._create_repeated_message_field(message_field.containing_type.name,
                                                       message_field.message_type.name, message_field.name)
        elif Meta._is_repeated_field(message_field):
            return self._create_repeated_field()
        elif Meta._is_message_field(message_field):
            if message_field.message_type.name == 'Timestamp':
                return self._create_timestamp_field()
            elif message_field.message_type.name == 'Any':
                return self._create_protobuf_any_field()
            else:
                return self._create_message_field(message_field.containing_type.name, message_field.message_type.name,
                                                  message_field.name)
        else:
            return self._create_generic_field(message_field_type)

    @staticmethod
    def _is_message_field(field_descriptor):
        return field_descriptor.message_type is not None

    @staticmethod
    def _is_repeated_field(field_descriptor):
        return field_descriptor.label == field_descriptor.LABEL_REPEATED

    @staticmethod
    def _is_repeated_message_field(field_descriptor):
        """
        Checks if a given field is a protobuf repeated field.
        :param field_descriptor: protobuf field descriptor
        :return: bool
        """
        return Meta._is_repeated_field(field_descriptor) and Meta._is_message_field(field_descriptor)

    @staticmethod
    def _is_map_field(field_descriptor):
        """
        Checks if a given field is a protobuf map to native field (map<int/float/str/etc., int/float/str/etc.>).
        :param field_descriptor: protobuf field descriptor
        :return: bool
        """
        return field_descriptor.message_type is not None and set(
            field_descriptor.message_type.fields_by_name.keys()) == {'key', 'value'}

    @staticmethod
    def _is_message_map_field(field_descriptor):
        """
        Checks if a given field is a protobuf map to message field (map<int/float/str/etc., Message>).
        :param field_descriptor: protobuf field descriptor
        :return: bool
        """
        return Meta._is_map_field(field_descriptor) and Meta._is_message_field(
            field_descriptor.message_type.fields_by_name['value'])

    def _create_generic_field(self, type_):
        """
        Creates a django field of the type that is defined in `pb_auto_field_type_mapping`.
        :param type_: Protobuf field type.
        :return: Django field.
        """
        field_type = self.pb_auto_field_type_mapping[type_]
        return field_type(null=True)

    def _create_timestamp_field(self):
        field_type = self.pb_auto_field_type_mapping[fields.PB_FIELD_TYPE_TIMESTAMP]
        return field_type()

    def _create_map_field(self):
        field_type = self.pb_auto_field_type_mapping[fields.PB_FIELD_TYPE_MAP]
        return field_type()

    def _create_repeated_field(self):
        field_type = self.pb_auto_field_type_mapping[fields.PB_FIELD_TYPE_REPEATED]
        return field_type()

    def _create_protobuf_any_field(self):
        field_type = self.pb_auto_field_type_mapping[fields.PB_FIELD_TYPE_MESSAGE_ANY]
        return field_type()

    def _create_message_field(self, own_type, related_type, field_name):
        field_type = self.pb_auto_field_type_mapping[fields.PB_FIELD_TYPE_MESSAGE]
        return field_type(to=related_type, related_name='%s_%s' % (own_type, field_name),
                          on_delete=models.deletion.CASCADE, null=True)

    def _create_repeated_message_field(self, own_type, related_type, field_name):
        """
        Creates a django relation that mimics a repeated message field.
        :param own_type: Name of the message that contains this field.
        :param related_type: Name of the message with which the relation is established.
        :param field_name: Name of the created field.
        :return: RepeatedMessageField
        """
        field_type = self.pb_auto_field_type_mapping[fields.PB_FIELD_TYPE_REPEATED_MESSAGE]
        return field_type(to=related_type, related_name='%s_%s' % (own_type, field_name))

    def _create_message_map_field(self, own_type, related_type, field_name):
        """
        Creates a django relation that mimics a scalar to message map field.
        :param own_type: Name of the message that contains this field.
        :param related_type: Name of the message with which the relation is established.
        :param field_name: Name of the created field.
        :return: MapToMessageField
        """
        field_type = self.pb_auto_field_type_mapping[fields.PB_FIELD_TYPE_MESSAGE_MAP]
        return field_type(to=related_type, related_name='%s_%s' % (own_type, field_name))

class ProtoBufMixin(six.with_metaclass(Meta, models.Model)):
    """This is mixin for model.Model.
    By setting attribute ``pb_model``, you can specify target ProtoBuf Message
    to handle django model.

    By settings attribute ``pb_2_dj_field_map``, you can mapping field from
    ProtoBuf to Django to handle schema migrations and message field changes
    """

    class Meta:
        abstract = True

    pb_model = None
    pb_2_dj_fields = []  # list of pb field names that are mapped, special case pb_2_dj_fields = '__all__'
    pb_2_dj_field_map = {}  # pb field in keys, dj field in value

    # defaults for models.DateTimeField and models.UUIDField
    # these serializers would be overwrited by definition in pb_2_dj_field_serializers if any
    _pb_2_dj_default_field_serializers = {
        models.DateTimeField: (fields._datetimefield_to_pb,
                               fields._datetimefield_from_pb),
        models.UUIDField: (fields._uuid_to_pb,
                           fields._uuid_from_pb),
    }

    pb_2_dj_field_serializers = {} # dj field in key, serializer function pairs in value

    # auto gen field defaults
    # mapping would be overwrited by definition in pb_auto_field_type_mapping if any
    _pb_auto_field_type_mapping = {
        FieldDescriptor.TYPE_DOUBLE: models.FloatField,
        FieldDescriptor.TYPE_FLOAT: models.FloatField,
        FieldDescriptor.TYPE_INT64: models.BigIntegerField,
        FieldDescriptor.TYPE_UINT64: models.BigIntegerField,
        FieldDescriptor.TYPE_INT32: models.IntegerField,
        FieldDescriptor.TYPE_FIXED64: models.DecimalField,
        FieldDescriptor.TYPE_FIXED32: models.DecimalField,
        FieldDescriptor.TYPE_BOOL: models.NullBooleanField,
        FieldDescriptor.TYPE_STRING: models.TextField,
        FieldDescriptor.TYPE_BYTES: models.BinaryField,
        FieldDescriptor.TYPE_UINT32: models.PositiveIntegerField,
        FieldDescriptor.TYPE_ENUM: models.IntegerField,
        FieldDescriptor.TYPE_SFIXED32: models.DecimalField,
        FieldDescriptor.TYPE_SFIXED64: models.DecimalField,
        FieldDescriptor.TYPE_SINT32: models.IntegerField,
        FieldDescriptor.TYPE_SINT64: models.BigIntegerField,
        fields.PB_FIELD_TYPE_TIMESTAMP: models.DateTimeField,
        fields.PB_FIELD_TYPE_REPEATED: fields.ArrayField,
        fields.PB_FIELD_TYPE_MAP: fields.MapField,
        fields.PB_FIELD_TYPE_MESSAGE: models.ForeignKey,
        fields.PB_FIELD_TYPE_REPEATED_MESSAGE: fields.RepeatedMessageField,
        fields.PB_FIELD_TYPE_MESSAGE_MAP: fields.MessageMapField,
        fields.PB_FIELD_TYPE_MESSAGE_ANY: fields.ProtoBufAnyField,
    }  # pb field type in key, dj field type in value
    """
    {ProtoBuf-field-name: Django-field-name} key-value pair mapping to handle
    schema migration or any model changes.
    """

    pb_auto_field_type_mapping = {}

    default_serializers = (fields._defaultfield_to_pb, fields._defaultfield_from_pb)

    def __init__(self, *args, **kwargs):
        super(ProtoBufMixin, self).__init__(*args, **kwargs)
        for m2m_field in self._meta.many_to_many:
            if issubclass(type(m2m_field), fields.ProtoBufFieldMixin):
                m2m_field.load(self)
        # TODO: also object.update

    def save(self, *args, **kwargs):
        super(ProtoBufMixin, self).save(*args, **kwargs)
        for m2m_field in self._meta.many_to_many:
            if issubclass(type(m2m_field), fields.ProtoBufFieldMixin):
                m2m_field.save(self)
        kwargs['force_insert'] = False
        super(ProtoBufMixin, self).save(*args, **kwargs)

    def _to_pb(self, _dj_field_name, _field, _pb_obj, _dj_fields,
               _dj_pb_field_map, depth):
        _dj_f_name = _dj_pb_field_map.get(_dj_field_name, _dj_field_name)
        if _dj_f_name not in _dj_fields:
            LOGGER.warning("No such django field: {}".format(_dj_f_name))
            return
        try:
            _dj_f_value = getattr(self, _dj_f_name)
            _dj_f_type = _dj_fields[_dj_f_name]
            if _dj_f_type.null and _dj_f_value is None:
                return

            # See if there's a custom serializer for this field relation or not
            field_serializers = self._get_serializers(type(_dj_f_type), _field)
            if field_serializers and field_serializers != self.default_serializers:
                self._value_to_protobuf(_pb_obj, _field, type(_dj_f_type), _dj_f_value)
            else:
                if _dj_f_type.is_relation and not issubclass(type(_dj_f_type), fields.ProtoBufFieldMixin):
                    self._relation_to_protobuf(_pb_obj, _field, _dj_f_type, _dj_f_value, depth)
                else:
                    self._value_to_protobuf(_pb_obj, _field, type(_dj_f_type), _dj_f_value)
        except AttributeError as e:
            LOGGER.error("Fail to serialize field: {} for {}. Error: {}".format(_dj_f_name, self._meta.model, e))
            raise DjangoPBModelError(
                "Can't serialize Model({})'s field: {}. Err: {}".format(_dj_f_name, self._meta.model, e))

    def _to_proto_recursively(self, _pb_obj, _pb_dj_field_map, _dj_fields,
                              depth):
        for _pb_field in _pb_obj.DESCRIPTOR.fields:
            _dj_field_name = _pb_dj_field_map.get(_pb_field.name, _pb_field.name)
            if isinstance(_dj_field_name, dict):
                self._to_proto_recursively(getattr(_pb_obj, _pb_field.name),
                                           _dj_field_name, _dj_fields,
                                           depth=depth)
            else:
                _field = _pb_obj.DESCRIPTOR.fields_by_name[_pb_field.name]
                self._to_pb(_dj_field_name, _field, _pb_obj, _dj_fields, _pb_dj_field_map, depth=depth)

    def to_pb(self, depth=None):
        """Convert django model to protobuf instance by pre-defined name

        :param depth: depth of relation been recursively converted. None means
            unlimited, 0 means no relation will be converted.

        :returns: ProtoBuf instance
        """
        _pb_obj = self.pb_model()

        # Mapping from proto field to Django field
        _pb_to_dj_mapping = self.pb_2_dj_field_map

        # Flat list of all Django fields
        _dj_fields = {f.name: f for f in self._meta.get_fields()}

        self._to_proto_recursively(_pb_obj, _pb_to_dj_mapping, _dj_fields, depth)

        LOGGER.info("Converted Protobuf [{}]: {}".format(
            _pb_obj.DESCRIPTOR.full_name, _pb_obj))
        return _pb_obj

    def _relation_to_protobuf(self, pb_obj, pb_field, dj_field_type,
                              dj_field_value, depth):
        """Handling relation to protobuf

        :param pb_obj: protobuf message obj which is return value of to_pb()
        :param pb_field: protobuf message field which is current processing field
        :param dj_field_type: Currently processing Django field type
        :param dj_field_value: Currently processing Django field value
        :param depth: depth of relation been recursively converted. None means
            unlimited, 0 means no relation will be converted.
        :returns: None

        """
        if isinstance(dj_field_type, ManyToOneRel):
            LOGGER.debug(
                "Django Relation field '{}' is reverse related, skipping".format(
                pb_field.name))
            return
        if depth is None or depth > 0:
            LOGGER.debug(
                "Django Relation field '{}', recursively serializing".format(
                    pb_field.name))
            if (depth is not None):
                LOGGER.debug('Current depth: {}'.format(depth))
        else:
            LOGGER.debug(
                "Django Relation field '{}', capped by depth, not converting".format(
                    pb_field.name))
            return

        next_depth = depth-1 if depth is not None else None
        if dj_field_type.many_to_many:
            self._m2m_to_protobuf(pb_obj, pb_field, dj_field_value, next_depth)
        else:
            getattr(pb_obj, pb_field.name).CopyFrom(dj_field_value.to_pb(
                depth=next_depth))

    def _m2m_to_protobuf(self, pb_obj, pb_field, dj_m2m_field, next_depth):
        """
        This is hook function from m2m field to protobuf. By default, we assume
        target message field is "repeated" nested message, ex:

        ```
        message M2M {
            int32 id = 1;
        }

        message Main {
            int32 id = 1;

            repeated M2M m2m = 2;
        }
        ```

        If this is not the format you expected, overwrite
        `_m2m_to_protobuf(self, pb_obj, pb_field, dj_field_value)` by yourself.

        :param pb_obj: intermediate-converting Protobuf obj, which would is return value of to_pb()
        :param pb_field: the Protobuf message field which supposed to assign after converting
        :param dj_m2mvalue: Django many_to_many field
        :param next_depth: the depth limit for the next level
        :returns: None

        """
        getattr(pb_obj, pb_field.name).extend(
            [_m2m.to_pb(depth=next_depth) for _m2m in dj_m2m_field.all()])

    def _get_serializers(self, dj_field_type, pb_field=None):
        """Getting the correct serializers for a field type

        :param dj_field_type: Currently processing django field type
        :returns: Tuple containing serialization func pair
        """
        if issubclass(dj_field_type, fields.ProtoBufFieldMixin):
            funcs = dj_field_type.to_pb, dj_field_type.from_pb
        else:
            defaults = self.default_serializers
            funcs = self.pb_2_dj_field_serializers.get(dj_field_type, None)
            if not funcs:
                if pb_field:
                    # Check by field name
                    funcs = self.pb_2_dj_field_serializers.get(pb_field.name, defaults)
                else:
                    funcs = defaults

        if len(funcs) != 2:
            LOGGER.warning(
                "Custom serializers require a pair of functions: {0} is misconfigured".
                    format(dj_field_type))
            return defaults

        return funcs

    def _value_to_protobuf(self, pb_obj, pb_field, dj_field_type, dj_field_value):
        """Handling value to protobuf

        :param pb_obj: protobuf message obj which is return value of to_pb()
        :param pb_field: protobuf message field which is current processing field
        :param dj_field_type: Currently processing django field type
        :param dj_field_value: Currently processing django field value
        :returns: None

        """
        s_funcs = self._get_serializers(dj_field_type, pb_field)
        s_funcs[0](pb_obj, pb_field, dj_field_value)

    def from_pb(self, _pb_obj):
        """Convert given protobuf obj to mixin Django model

        :returns: Django model instance
        """
        _dj_field_map = {f.name: f for f in self._meta.get_fields()}
        _pb_dj_field_map = self.pb_2_dj_field_map
        LOGGER.debug("ListFields() returns only fields which contain a value")
        self._from_pb_recursively(_dj_field_map, _pb_obj, _pb_dj_field_map)

        LOGGER.info("Converted Django model instance: {}".format(self))
        return self

    def _from_pb_recursively(self, _dj_field_map, _pb_obj, _pb_dj_field_map):
        # ListFields only returns fields with values
        for _f, _v in _pb_obj.ListFields():
            _dj_field_name = _pb_dj_field_map.get(_f.name)
            if hasattr(_v, "DESCRIPTOR") and _dj_field_name is not None:
                self._from_pb_recursively(_dj_field_map, _v, _dj_field_name)
            else:
                _dj_f_name = _dj_field_name if _dj_field_name is not None else _f.name
                self._from_pb(_dj_field_map, _f, _v, _dj_f_name)

    def _from_pb(self, _dj_field_map, _f, _v, _dj_f_name):
        _dj_f_type = _dj_field_map[_dj_f_name]

        field_serializers = self._get_serializers(type(_dj_f_type), _f)
        if field_serializers and field_serializers != self.default_serializers:
            self._protobuf_to_value(_dj_f_name, type(_dj_f_type), _f, _v)

        if _f.message_type is not None:
            dj_field = _dj_field_map[_dj_f_name]
            if dj_field.is_relation and not issubclass(type(dj_field), fields.ProtoBufFieldMixin):
                self._protobuf_to_relation(_dj_f_name, dj_field, _f, _v)
                return
        self._protobuf_to_value(_dj_f_name, type(_dj_f_type), _f, _v)

    def _protobuf_to_relation(self, dj_field_name, dj_field, pb_field,
                              pb_value):
        """Handling protobuf nested message to relation key

        :param dj_field_name: Currently target django field's name
        :param dj_field: Currently target django field
        :param pb_field: Currently processing protobuf message field
        :param pb_value: Currently processing protobuf message value
        :returns: None
        """
        LOGGER.debug("Django Relation Feild, deserializing Probobuf message")
        if dj_field.many_to_many:
            self._protobuf_to_m2m(dj_field_name, dj_field, pb_value)
            return

        if hasattr(dj_field, 'related_model'):
            # django > 1.8 compatible
            setattr(self, dj_field_name, dj_field.related_model().from_pb(pb_value))
        else:
            setattr(self, dj_field_name, dj_field.related.model().from_pb(pb_value))

    def _protobuf_to_m2m(self, dj_field_name, dj_field, pb_repeated_set):
        """
        This is hook function to handle repeated list to m2m field while converting
        from protobuf to django. By default, no operation is performed, which means
        you may query current relation if your converted django model instance has a valid PK.

        If you want to modify your database while converting on-the-fly, you may
        override the logic:

        ```
        from django.db import transaction

        ...

        class PBCompatibleModel(ProtoBufMixin, models.Model):

            def _repeated_to_m2m(self, dj_field, _pb_repeated_set):
                with transaction.atomic():
                    for item in _pb_repeated_set:
                        dj_field.get_or_create(pk=item.pk, defaults={....})

            ...

        ```

        :param dj_field: Django many_to_many field
        :param pb_repeated_set: the Protobuf message field which contains data that is going to be converted
        :returns: None

        """
        return

    def _protobuf_to_value(self, dj_field_name, dj_field_type, pb_field,
                           pb_value):
        """Handling protobuf singular value

        :param dj_field_name: Currently target django field's name
        :param dj_field_type: Currently proecessing django field type
        :param pb_field: Currently processing protobuf message field
        :param pb_value: Currently processing protobuf message value
        :returns: None
        """
        s_funcs = self._get_serializers(dj_field_type, pb_field)
        s_funcs[1](self, dj_field_name, pb_field, pb_value)



import datetime
import uuid

from django.test import TestCase
from django.db import models as dj_models

from google.protobuf.any_pb2 import Any
from google.protobuf.timestamp_pb2 import Timestamp
from google.protobuf.descriptor import FieldDescriptor

# Create your tests here.

from pb_model.models import ProtoBufMixin
from . import models, models_pb2


class ProtoBufConvertingTest(TestCase):

    def setUp(self):
        pass

    def test_single_model(self):
        relation_item = models.Relation.objects.create(num=10)
        relation_item2 = models.Relation()

        relation_item2.from_pb(relation_item.to_pb())

        self.assertEqual(relation_item.id, relation_item2.id,
                         msg="{}(src) != {}(target)".format(relation_item.id, relation_item2.id))
        self.assertEqual(relation_item.num, relation_item2.num,
                         msg="{}(src) != {}(target)".format(relation_item.num, relation_item2.num))

    def test_model_with_key(self):
        main_item = models.Main.objects.create(
            string_field='Hello world', integer_field=2017,
            float_field=3.14159, bool_field=True,
            choices_field=models.Main.OPT2,
            fk_field=models.Relation.objects.create(num=2018),
        )
        m2m_relations = [models.M2MRelation.objects.create(num=i+2000) for i in range(5)]
        for m2m in m2m_relations:
            main_item.m2m_field.add(m2m)

        main_item2 = models.Main()
        main_item2.from_pb(main_item.to_pb())

        self.assertEqual(main_item.id, main_item2.id,
                         msg="{}(src) != {}(target)".format(main_item.id, main_item2.id))
        self.assertEqual(main_item.string_field, main_item2.string_field,
                         msg="{}(src) != {}(target)".format(main_item.string_field, main_item2.string_field))
        self.assertEqual(main_item.integer_field, main_item2.integer_field,
                         msg="{}(src) != {}(target)".format(main_item.integer_field, main_item2.integer_field))
        self.assertAlmostEqual(main_item.float_field, main_item2.float_field, delta=1e-6,
                               msg="{}(src) != {}(target)".format(main_item.float_field, main_item2.float_field))
        self.assertEqual(main_item.bool_field, main_item2.bool_field,
                         msg="{}(src) != {}(target)".format(main_item.bool_field, main_item2.bool_field))
        self.assertEqual(main_item.choices_field, main_item2.choices_field,
                         msg="{}(src) != {}(target)".format(main_item.choices_field, main_item2.choices_field))

        time_diff = main_item.datetime_field - main_item2.datetime_field
        # convertion may affect 1 microsecond due to resolution difference
        # between Protobuf timestamp and Python datetime
        self.assertAlmostEqual(0, time_diff.total_seconds(), delta=1e-6,
                               msg="{}(src) != {}(target)".format(main_item.datetime_field, main_item2.datetime_field))

        self.assertEqual(main_item.fk_field.id, main_item2.fk_field.id,
                         msg="{}(src) != {}(target)".format(main_item.fk_field.id, main_item2.fk_field.id))
        self.assertEqual(main_item.fk_field.num, main_item2.fk_field.num,
                         msg="{}(src) != {}(target)".format(main_item.fk_field.num, main_item2.fk_field.num))

        self.assertListEqual(
            list(main_item.m2m_field.order_by('id').values_list('id', flat=True)),
            list(main_item2.m2m_field.order_by('id').values_list('id', flat=True)),
            msg="{}(src) != {}(target)".format(
                main_item.m2m_field.order_by('id').values_list('id', flat=True),
                main_item2.m2m_field.order_by('id').values_list('id', flat=True))
        )

        main_item2.save()
        main_item2 = models.Main.objects.get()
        assert main_item.to_pb() == main_item2.to_pb()

    def test_inheritance(self):
        class Parent(ProtoBufMixin, dj_models.Model):
            pb_model = models_pb2.Root
            pb_2_dj_fields = ['uint32_field']
            pb_2_dj_field_map = {'uint32_field': 'uint32_field_renamed'}
            pb_auto_field_type_mapping = {
                FieldDescriptor.TYPE_UINT32: dj_models.IntegerField
            }

        class Child(Parent):
            pb_model = models_pb2.Root
            pb_2_dj_fields = ['uint64_field']
            pb_2_dj_field_map = {'uint64_field': 'uint64_field_renamed'}
            pb_auto_field_type_mapping = {
                FieldDescriptor.TYPE_UINT32: dj_models.FloatField,
                FieldDescriptor.TYPE_UINT64: dj_models.IntegerField
            }

        assert ProtoBufMixin.pb_auto_field_type_mapping[FieldDescriptor.TYPE_UINT32] is dj_models.PositiveIntegerField
        assert Parent.pb_auto_field_type_mapping[FieldDescriptor.TYPE_UINT32] is dj_models.IntegerField

        assert {f.name for f in Parent._meta.get_fields()} == {'child', 'id', 'uint32_field_renamed'}
        assert type(Parent._meta.get_field('uint32_field_renamed')) is dj_models.IntegerField

        assert {f.name for f in Child._meta.get_fields()} == {'parent_ptr', 'id', 'uint32_field_renamed', 'uint64_field_renamed'}
        assert type(Child._meta.get_field('uint32_field_renamed')) is dj_models.IntegerField
        assert type(Child._meta.get_field('uint64_field_renamed')) is dj_models.IntegerField

    def test_custom_serializer(self):
        """
        Default serialization strategies can be overriden
        """

        def serializer(pb_obj, pb_field, dj_value):
            """
            Serialize NativeRelation as a repeated int32
            """
            getattr(pb_obj, 'foreign_field').extend([dj_value.first, dj_value.second, dj_value.third])


        def deserializer(instance, dj_field_name, pb_field, pb_value):
            setattr(instance, 'foreign_field',
                    NativeRelation(
                        first=pb_value[0],
                        second=pb_value[1],
                        third=pb_value[2]
                    ))

        test_uuid = uuid.UUID('acf9d4c5-335c-4ad7-9e60-32e7306ea7c0')

        def _test_uuid_to_pb(pb_obj, pb_field, dj_field_value):
            # always set to test_uuid at test
            setattr(pb_obj, pb_field.name, test_uuid.hex)

        def _test_uuid_from_pb(instance, dj_field_name, pb_field, pb_value):
            setattr(instance, dj_field_name, uuid.UUID(pb_value))

        # This is a relation type that's not ProtoBuf enabled
        class NativeRelation(dj_models.Model):
            first = dj_models.IntegerField()
            second = dj_models.IntegerField()
            third = dj_models.IntegerField()


        class Model(ProtoBufMixin, dj_models.Model):
            pb_model = models_pb2.Root
            pb_2_dj_fields = ['foreign_field']
            pb_2_dj_field_serializers = {
                'foreign_field': (serializer, deserializer),
                dj_models.UUIDField: (_test_uuid_to_pb, _test_uuid_from_pb),
            }
            foreign_field = dj_models.ForeignKey(NativeRelation, on_delete=dj_models.DO_NOTHING)
            uuid_field = dj_models.UUIDField()

        _in = Model(
            foreign_field=NativeRelation(first=123, second=456, third=789),
            uuid_field=uuid.uuid4(),
        )

        out = Model().from_pb(_in.to_pb())

        assert out.foreign_field.first == 123
        assert out.foreign_field.second == 456
        assert out.foreign_field.third == 789

        assert _in.uuid_field != test_uuid
        assert out.uuid_field == test_uuid


    def test_auto_fields(self):
        timestamp = Timestamp()
        timestamp.FromDatetime(datetime.datetime.now())

        _any = Any()
        _any.Pack(timestamp)

        pb_object = models_pb2.Root(
            uint32_field=1234,
            uint64_field=123,
            int64_field=123,
            float_field=12.3,
            double_field=12.3,
            string_field='123',
            bytes_field=b'123',
            bool_field=True,
            uuid_field=str(uuid.uuid4()),
            any_field=_any,
            enum_field=1,
            timestamp_field=timestamp,
            repeated_uint32_field=[1, 2, 3],
            map_string_to_string_field={'qwe': 'asd'},

            message_field=models_pb2.Root.Embedded(data=123),
            repeated_message_field=[models_pb2.Root.Embedded(data=123), models_pb2.Root.Embedded(data=456), models_pb2.Root.Embedded(data=789)],
            map_string_to_message_field={'qwe': models_pb2.Root.Embedded(data=123), 'asd': models_pb2.Root.Embedded(data=456)},

            list_field_option=models_pb2.Root.ListWrapper(data=['qwe', 'asd', 'zxc']),

            inlineField=models_pb2.Root.InlineEmbedding(
                data="qwerty",
                doublyNestedField=models_pb2.Root.InlineEmbedding.NestedEmbedding(
                    data="qqwwee",
                ),
            ),
        )

        dj_object = models.Root()
        dj_object.from_pb(pb_object)
        dj_object.message_field.save()
        dj_object.message_field = dj_object.message_field
        for m in dj_object.repeated_message_field:
            m.save()
        for m in dj_object.map_string_to_message_field.values():
            m.save()
        dj_object.list_field_option.save()
        dj_object.list_field_option = dj_object.list_field_option
        dj_object.save()

        dj_object_from_db = models.Root.objects.get()
        assert [o.data for o in dj_object_from_db.repeated_message_field] == [123, 456, 789]
        assert {k: o.data for k, o in dj_object_from_db.map_string_to_message_field.items()} == {'qwe': 123, 'asd': 456}
        assert dj_object_from_db.uint32_field_renamed == pb_object.uint32_field
        assert dj_object_from_db.any_field == _any
        result = dj_object_from_db.to_pb()
        assert pb_object == result

        # test proxy model with auto field
        proxy_object_from_db = models.Proxy.objects.get()
        assert [o.data for o in proxy_object_from_db.repeated_message_field] == [123, 456, 789]
        assert {k: o.data for k, o in proxy_object_from_db.map_string_to_message_field.items()} == {'qwe': 123, 'asd': 456}
        assert proxy_object_from_db.uint32_field_renamed == pb_object.uint32_field
        assert proxy_object_from_db.any_field == _any
        result = proxy_object_from_db.to_pb()
        assert pb_object == result

    def test_relation_depth_limit(self):
        deepter_relation_item = models.DeeperRelation.objects.create(num=2)
        relation_item = models.Relation.objects.create(
            num=1, deeper_relation=deepter_relation_item)
        main_item = models.Main.objects.create(
            string_field='Hello world', integer_field=0,
            float_field=3.14159, bool_field=True,
            choices_field=models.Main.OPT2,
            fk_field=relation_item
        )

        # Verify all relation is converted when no depth provided.
        unlimited_main_proto = main_item.to_pb()

        self.assertEqual(unlimited_main_proto.fk_field.id, relation_item.id,
                         msg="{}(src) != {}(target)".format(
                         unlimited_main_proto.fk_field.id, relation_item.id))
        self.assertEqual(unlimited_main_proto.fk_field.num, relation_item.num,
                         msg="{}(src) != {}(target)".format(
                         unlimited_main_proto.fk_field.num, relation_item.num))
        self.assertEqual(unlimited_main_proto.fk_field.deeper_relation.id,
                         deepter_relation_item.id,
                         msg="{}(src) != {}(target)".format(
                         unlimited_main_proto.fk_field.deeper_relation.id,
                         deepter_relation_item.id))
        self.assertEqual(unlimited_main_proto.fk_field.deeper_relation.num,
                         deepter_relation_item.num,
                         msg="{}(src) != {}(target)".format(
                         unlimited_main_proto.fk_field.deeper_relation.num,
                         deepter_relation_item.num))


        # Verify no relation is converted when depth = 0.
        cap_0_main_proto = main_item.to_pb(depth=0)
        # proto3 does not support `hasField`, check value not equal instead.
        self.assertNotEqual(cap_0_main_proto.fk_field.id,
                            relation_item.id,
                            msg="{}(src) == {}(target)".format(
                                cap_0_main_proto.fk_field.id,
                                relation_item.id))
        self.assertNotEqual(cap_0_main_proto.fk_field.num,
                            relation_item.num,
                            msg="{}(src) == {}(target)".format(
                                cap_0_main_proto.fk_field.num,
                                relation_item.num))
        self.assertNotEqual(
            cap_0_main_proto.fk_field.deeper_relation.id,
            deepter_relation_item.id,
            msg="{}(src) == {}(target)".format(
                cap_0_main_proto.fk_field.deeper_relation.id,
                deepter_relation_item.id))
        self.assertNotEqual(
            cap_0_main_proto.fk_field.deeper_relation.num,
            deepter_relation_item.num,
            msg="{}(src) == {}(target)".format(
                cap_0_main_proto.fk_field.deeper_relation.num,
                deepter_relation_item.num))

        # Verify only 1 level relation is converted when depth = 1.
        cap_1_main_proto = main_item.to_pb(depth=1)
        # proto3 does not support `hasField`, check value not equal instead.
        self.assertEqual(cap_1_main_proto.fk_field.id,
                         relation_item.id,
                         msg="{}(src) != {}(target)".format(
                            cap_1_main_proto.fk_field.id,
                            relation_item.id))
        self.assertEqual(cap_1_main_proto.fk_field.num,
                         relation_item.num,
                         msg="{}(src) != {}(target)".format(
                            cap_1_main_proto.fk_field.num,
                            relation_item.num))
        self.assertNotEqual(
            cap_1_main_proto.fk_field.deeper_relation.id,
            deepter_relation_item.id,
            msg="{}(src) == {}(target)".format(
                cap_1_main_proto.fk_field.deeper_relation.id,
                deepter_relation_item.id))
        self.assertNotEqual(
            cap_1_main_proto.fk_field.deeper_relation.num,
            deepter_relation_item.num,
            msg="{}(src) == {}(target)".format(
                cap_1_main_proto.fk_field.deeper_relation.num,
                deepter_relation_item.num))

    def test_reverse_relation(self):
        deeper_relation_item = models.DeeperRelation.objects.create(num=2)
        relation_item1 = models.Relation.objects.create(
            num=1, deeper_relation=deeper_relation_item)
        relation_item2 = models.Relation.objects.create(
            num=2, deeper_relation=deeper_relation_item)

        test_proto = deeper_relation_item.to_pb()



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: models.proto

from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import any_pb2 as google_dot_protobuf_dot_any__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='models.proto',
  package='models',
  syntax='proto3',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x0cmodels.proto\x12\x06models\x1a\x19google/protobuf/any.proto\x1a\x1fgoogle/protobuf/timestamp.proto\"N\n\x0e\x44\x65\x65perRelation\x12\n\n\x02id\x18\x01 \x01(\x05\x12\x0b\n\x03num\x18\x02 \x01(\x05\x12#\n\trelations\x18\x03 \x03(\x0b\x32\x10.models.Relation\"T\n\x08Relation\x12\n\n\x02id\x18\x01 \x01(\x05\x12\x0b\n\x03num\x18\x02 \x01(\x05\x12/\n\x0f\x64\x65\x65per_relation\x18\x03 \x01(\x0b\x32\x16.models.DeeperRelation\"&\n\x0bM2MRelation\x12\n\n\x02id\x18\x01 \x01(\x05\x12\x0b\n\x03num\x18\x02 \x01(\x05\"\xc8\x02\n\x04Main\x12\n\n\x02id\x18\x01 \x01(\x05\x12\x14\n\x0cstring_field\x18\x02 \x01(\t\x12\x15\n\rinteger_field\x18\x03 \x01(\x05\x12\x13\n\x0b\x66loat_field\x18\x04 \x01(\x02\x12+\n\rchoices_field\x18\x05 \x01(\x0e\x32\x14.models.Main.Options\x12\"\n\x08\x66k_field\x18\x06 \x01(\x0b\x32\x10.models.Relation\x12&\n\tm2m_field\x18\x07 \x03(\x0b\x32\x13.models.M2MRelation\x12\x12\n\nbool_field\x18\x08 \x01(\x08\x12\x32\n\x0e\x64\x61tetime_field\x18\t \x01(\x0b\x32\x1a.google.protobuf.Timestamp\"1\n\x07Options\x12\x08\n\x04OPT0\x10\x00\x12\x08\n\x04OPT1\x10\x01\x12\x08\n\x04OPT2\x10\x02\x12\x08\n\x04OPT3\x10\x03\"\xc0\n\n\x04Root\x12\x14\n\x0cuint32_field\x18\x01 \x01(\r\x12\x13\n\x0bint32_field\x18\x02 \x01(\x05\x12\x14\n\x0cuint64_field\x18\x03 \x01(\x04\x12\x13\n\x0bint64_field\x18\x04 \x01(\x03\x12\x13\n\x0b\x66loat_field\x18\x05 \x01(\x02\x12\x14\n\x0c\x64ouble_field\x18\x06 \x01(\x01\x12\x14\n\x0cstring_field\x18\x07 \x01(\t\x12\x13\n\x0b\x62ytes_field\x18\x08 \x01(\x0c\x12\x12\n\nbool_field\x18\t \x01(\x08\x12 \n\nenum_field\x18\n \x01(\x0e\x32\x0c.models.Enum\x12\x33\n\x0ftimestamp_field\x18\x0b \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x12\n\nuuid_field\x18\x0c \x01(\t\x12\x1d\n\x15repeated_uint32_field\x18\r \x03(\r\x12\x1d\n\x15repeated_string_field\x18\x0e \x03(\t\x12\x1d\n\x15repeated_double_field\x18\x0f \x03(\x01\x12L\n\x1amap_string_to_string_field\x18\x10 \x03(\x0b\x32(.models.Root.MapStringToStringFieldEntry\x12,\n\rmessage_field\x18\x11 \x01(\x0b\x32\x15.models.Root.Embedded\x12\x35\n\x16repeated_message_field\x18\x12 \x03(\x0b\x32\x15.models.Root.Embedded\x12N\n\x1bmap_string_to_message_field\x18\x13 \x03(\x0b\x32).models.Root.MapStringToMessageFieldEntry\x12\x35\n\x11list_field_option\x18\x14 \x01(\x0b\x32\x18.models.Root.ListWrapperH\x00\x12\x33\n\x10map_field_option\x18\x15 \x01(\x0b\x32\x17.models.Root.MapWrapperH\x00\x12\'\n\tany_field\x18\x16 \x01(\x0b\x32\x14.google.protobuf.Any\x12\x15\n\rforeign_field\x18\x1e \x03(\x05\x12\x31\n\x0binlineField\x18( \x01(\x0b\x32\x1c.models.Root.InlineEmbedding\x1a=\n\x1bMapStringToStringFieldEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1aU\n\x1cMapStringToMessageFieldEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12$\n\x05value\x18\x02 \x01(\x0b\x32\x15.models.Root.Embedded:\x02\x38\x01\x1a\x18\n\x08\x45mbedded\x12\x0c\n\x04\x64\x61ta\x18\x01 \x01(\x05\x1a\x1b\n\x0bListWrapper\x12\x0c\n\x04\x64\x61ta\x18\x01 \x03(\t\x1aj\n\nMapWrapper\x12/\n\x04\x64\x61ta\x18\x01 \x03(\x0b\x32!.models.Root.MapWrapper.DataEntry\x1a+\n\tDataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a\x89\x01\n\x0fInlineEmbedding\x12\x0c\n\x04\x64\x61ta\x18\x01 \x01(\t\x12G\n\x11\x64oublyNestedField\x18\x02 \x01(\x0b\x32,.models.Root.InlineEmbedding.NestedEmbedding\x1a\x1f\n\x0fNestedEmbedding\x12\x0c\n\x04\x64\x61ta\x18\x01 \x01(\tB\t\n\x07options*3\n\x04\x45num\x12\x0f\n\x0b\x45num_NOTSET\x10\x00\x12\x0c\n\x08\x45num_ONE\x10\x01\x12\x0c\n\x08\x45num_TWO\x10\x02\x62\x06proto3'
  ,
  dependencies=[google_dot_protobuf_dot_any__pb2.DESCRIPTOR,google_dot_protobuf_dot_timestamp__pb2.DESCRIPTOR,])

_ENUM = _descriptor.EnumDescriptor(
  name='Enum',
  full_name='models.Enum',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='Enum_NOTSET', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='Enum_ONE', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='Enum_TWO', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=1968,
  serialized_end=2019,
)
_sym_db.RegisterEnumDescriptor(_ENUM)

Enum = enum_type_wrapper.EnumTypeWrapper(_ENUM)
Enum_NOTSET = 0
Enum_ONE = 1
Enum_TWO = 2


_MAIN_OPTIONS = _descriptor.EnumDescriptor(
  name='Options',
  full_name='models.Main.Options',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='OPT0', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='OPT1', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='OPT2', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='OPT3', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=570,
  serialized_end=619,
)
_sym_db.RegisterEnumDescriptor(_MAIN_OPTIONS)


_DEEPERRELATION = _descriptor.Descriptor(
  name='DeeperRelation',
  full_name='models.DeeperRelation',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='models.DeeperRelation.id', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='num', full_name='models.DeeperRelation.num', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='relations', full_name='models.DeeperRelation.relations', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=84,
  serialized_end=162,
)


_RELATION = _descriptor.Descriptor(
  name='Relation',
  full_name='models.Relation',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='models.Relation.id', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='num', full_name='models.Relation.num', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='deeper_relation', full_name='models.Relation.deeper_relation', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=164,
  serialized_end=248,
)


_M2MRELATION = _descriptor.Descriptor(
  name='M2MRelation',
  full_name='models.M2MRelation',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='models.M2MRelation.id', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='num', full_name='models.M2MRelation.num', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=250,
  serialized_end=288,
)


_MAIN = _descriptor.Descriptor(
  name='Main',
  full_name='models.Main',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='models.Main.id', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='string_field', full_name='models.Main.string_field', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='integer_field', full_name='models.Main.integer_field', index=2,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='float_field', full_name='models.Main.float_field', index=3,
      number=4, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='choices_field', full_name='models.Main.choices_field', index=4,
      number=5, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='fk_field', full_name='models.Main.fk_field', index=5,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='m2m_field', full_name='models.Main.m2m_field', index=6,
      number=7, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='bool_field', full_name='models.Main.bool_field', index=7,
      number=8, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='datetime_field', full_name='models.Main.datetime_field', index=8,
      number=9, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _MAIN_OPTIONS,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=291,
  serialized_end=619,
)


_ROOT_MAPSTRINGTOSTRINGFIELDENTRY = _descriptor.Descriptor(
  name='MapStringToStringFieldEntry',
  full_name='models.Root.MapStringToStringFieldEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='models.Root.MapStringToStringFieldEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='models.Root.MapStringToStringFieldEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'8\001',
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1504,
  serialized_end=1565,
)

_ROOT_MAPSTRINGTOMESSAGEFIELDENTRY = _descriptor.Descriptor(
  name='MapStringToMessageFieldEntry',
  full_name='models.Root.MapStringToMessageFieldEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='models.Root.MapStringToMessageFieldEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='models.Root.MapStringToMessageFieldEntry.value', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'8\001',
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1567,
  serialized_end=1652,
)

_ROOT_EMBEDDED = _descriptor.Descriptor(
  name='Embedded',
  full_name='models.Root.Embedded',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='data', full_name='models.Root.Embedded.data', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1654,
  serialized_end=1678,
)

_ROOT_LISTWRAPPER = _descriptor.Descriptor(
  name='ListWrapper',
  full_name='models.Root.ListWrapper',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='data', full_name='models.Root.ListWrapper.data', index=0,
      number=1, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1680,
  serialized_end=1707,
)

_ROOT_MAPWRAPPER_DATAENTRY = _descriptor.Descriptor(
  name='DataEntry',
  full_name='models.Root.MapWrapper.DataEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='models.Root.MapWrapper.DataEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='models.Root.MapWrapper.DataEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'8\001',
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1772,
  serialized_end=1815,
)

_ROOT_MAPWRAPPER = _descriptor.Descriptor(
  name='MapWrapper',
  full_name='models.Root.MapWrapper',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='data', full_name='models.Root.MapWrapper.data', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_ROOT_MAPWRAPPER_DATAENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1709,
  serialized_end=1815,
)

_ROOT_INLINEEMBEDDING_NESTEDEMBEDDING = _descriptor.Descriptor(
  name='NestedEmbedding',
  full_name='models.Root.InlineEmbedding.NestedEmbedding',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='data', full_name='models.Root.InlineEmbedding.NestedEmbedding.data', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1924,
  serialized_end=1955,
)

_ROOT_INLINEEMBEDDING = _descriptor.Descriptor(
  name='InlineEmbedding',
  full_name='models.Root.InlineEmbedding',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='data', full_name='models.Root.InlineEmbedding.data', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='doublyNestedField', full_name='models.Root.InlineEmbedding.doublyNestedField', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_ROOT_INLINEEMBEDDING_NESTEDEMBEDDING, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1818,
  serialized_end=1955,
)

_ROOT = _descriptor.Descriptor(
  name='Root',
  full_name='models.Root',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='uint32_field', full_name='models.Root.uint32_field', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='int32_field', full_name='models.Root.int32_field', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='uint64_field', full_name='models.Root.uint64_field', index=2,
      number=3, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='int64_field', full_name='models.Root.int64_field', index=3,
      number=4, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='float_field', full_name='models.Root.float_field', index=4,
      number=5, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='double_field', full_name='models.Root.double_field', index=5,
      number=6, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='string_field', full_name='models.Root.string_field', index=6,
      number=7, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='bytes_field', full_name='models.Root.bytes_field', index=7,
      number=8, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='bool_field', full_name='models.Root.bool_field', index=8,
      number=9, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='enum_field', full_name='models.Root.enum_field', index=9,
      number=10, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='timestamp_field', full_name='models.Root.timestamp_field', index=10,
      number=11, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='uuid_field', full_name='models.Root.uuid_field', index=11,
      number=12, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='repeated_uint32_field', full_name='models.Root.repeated_uint32_field', index=12,
      number=13, type=13, cpp_type=3, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='repeated_string_field', full_name='models.Root.repeated_string_field', index=13,
      number=14, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='repeated_double_field', full_name='models.Root.repeated_double_field', index=14,
      number=15, type=1, cpp_type=5, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='map_string_to_string_field', full_name='models.Root.map_string_to_string_field', index=15,
      number=16, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='message_field', full_name='models.Root.message_field', index=16,
      number=17, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='repeated_message_field', full_name='models.Root.repeated_message_field', index=17,
      number=18, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='map_string_to_message_field', full_name='models.Root.map_string_to_message_field', index=18,
      number=19, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='list_field_option', full_name='models.Root.list_field_option', index=19,
      number=20, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='map_field_option', full_name='models.Root.map_field_option', index=20,
      number=21, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='any_field', full_name='models.Root.any_field', index=21,
      number=22, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='foreign_field', full_name='models.Root.foreign_field', index=22,
      number=30, type=5, cpp_type=1, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='inlineField', full_name='models.Root.inlineField', index=23,
      number=40, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_ROOT_MAPSTRINGTOSTRINGFIELDENTRY, _ROOT_MAPSTRINGTOMESSAGEFIELDENTRY, _ROOT_EMBEDDED, _ROOT_LISTWRAPPER, _ROOT_MAPWRAPPER, _ROOT_INLINEEMBEDDING, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='options', full_name='models.Root.options',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=622,
  serialized_end=1966,
)

_DEEPERRELATION.fields_by_name['relations'].message_type = _RELATION
_RELATION.fields_by_name['deeper_relation'].message_type = _DEEPERRELATION
_MAIN.fields_by_name['choices_field'].enum_type = _MAIN_OPTIONS
_MAIN.fields_by_name['fk_field'].message_type = _RELATION
_MAIN.fields_by_name['m2m_field'].message_type = _M2MRELATION
_MAIN.fields_by_name['datetime_field'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_MAIN_OPTIONS.containing_type = _MAIN
_ROOT_MAPSTRINGTOSTRINGFIELDENTRY.containing_type = _ROOT
_ROOT_MAPSTRINGTOMESSAGEFIELDENTRY.fields_by_name['value'].message_type = _ROOT_EMBEDDED
_ROOT_MAPSTRINGTOMESSAGEFIELDENTRY.containing_type = _ROOT
_ROOT_EMBEDDED.containing_type = _ROOT
_ROOT_LISTWRAPPER.containing_type = _ROOT
_ROOT_MAPWRAPPER_DATAENTRY.containing_type = _ROOT_MAPWRAPPER
_ROOT_MAPWRAPPER.fields_by_name['data'].message_type = _ROOT_MAPWRAPPER_DATAENTRY
_ROOT_MAPWRAPPER.containing_type = _ROOT
_ROOT_INLINEEMBEDDING_NESTEDEMBEDDING.containing_type = _ROOT_INLINEEMBEDDING
_ROOT_INLINEEMBEDDING.fields_by_name['doublyNestedField'].message_type = _ROOT_INLINEEMBEDDING_NESTEDEMBEDDING
_ROOT_INLINEEMBEDDING.containing_type = _ROOT
_ROOT.fields_by_name['enum_field'].enum_type = _ENUM
_ROOT.fields_by_name['timestamp_field'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_ROOT.fields_by_name['map_string_to_string_field'].message_type = _ROOT_MAPSTRINGTOSTRINGFIELDENTRY
_ROOT.fields_by_name['message_field'].message_type = _ROOT_EMBEDDED
_ROOT.fields_by_name['repeated_message_field'].message_type = _ROOT_EMBEDDED
_ROOT.fields_by_name['map_string_to_message_field'].message_type = _ROOT_MAPSTRINGTOMESSAGEFIELDENTRY
_ROOT.fields_by_name['list_field_option'].message_type = _ROOT_LISTWRAPPER
_ROOT.fields_by_name['map_field_option'].message_type = _ROOT_MAPWRAPPER
_ROOT.fields_by_name['any_field'].message_type = google_dot_protobuf_dot_any__pb2._ANY
_ROOT.fields_by_name['inlineField'].message_type = _ROOT_INLINEEMBEDDING
_ROOT.oneofs_by_name['options'].fields.append(
  _ROOT.fields_by_name['list_field_option'])
_ROOT.fields_by_name['list_field_option'].containing_oneof = _ROOT.oneofs_by_name['options']
_ROOT.oneofs_by_name['options'].fields.append(
  _ROOT.fields_by_name['map_field_option'])
_ROOT.fields_by_name['map_field_option'].containing_oneof = _ROOT.oneofs_by_name['options']
DESCRIPTOR.message_types_by_name['DeeperRelation'] = _DEEPERRELATION
DESCRIPTOR.message_types_by_name['Relation'] = _RELATION
DESCRIPTOR.message_types_by_name['M2MRelation'] = _M2MRELATION
DESCRIPTOR.message_types_by_name['Main'] = _MAIN
DESCRIPTOR.message_types_by_name['Root'] = _ROOT
DESCRIPTOR.enum_types_by_name['Enum'] = _ENUM
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

DeeperRelation = _reflection.GeneratedProtocolMessageType('DeeperRelation', (_message.Message,), {
  'DESCRIPTOR' : _DEEPERRELATION,
  '__module__' : 'models_pb2'
  # @@protoc_insertion_point(class_scope:models.DeeperRelation)
  })
_sym_db.RegisterMessage(DeeperRelation)

Relation = _reflection.GeneratedProtocolMessageType('Relation', (_message.Message,), {
  'DESCRIPTOR' : _RELATION,
  '__module__' : 'models_pb2'
  # @@protoc_insertion_point(class_scope:models.Relation)
  })
_sym_db.RegisterMessage(Relation)

M2MRelation = _reflection.GeneratedProtocolMessageType('M2MRelation', (_message.Message,), {
  'DESCRIPTOR' : _M2MRELATION,
  '__module__' : 'models_pb2'
  # @@protoc_insertion_point(class_scope:models.M2MRelation)
  })
_sym_db.RegisterMessage(M2MRelation)

Main = _reflection.GeneratedProtocolMessageType('Main', (_message.Message,), {
  'DESCRIPTOR' : _MAIN,
  '__module__' : 'models_pb2'
  # @@protoc_insertion_point(class_scope:models.Main)
  })
_sym_db.RegisterMessage(Main)

Root = _reflection.GeneratedProtocolMessageType('Root', (_message.Message,), {

  'MapStringToStringFieldEntry' : _reflection.GeneratedProtocolMessageType('MapStringToStringFieldEntry', (_message.Message,), {
    'DESCRIPTOR' : _ROOT_MAPSTRINGTOSTRINGFIELDENTRY,
    '__module__' : 'models_pb2'
    # @@protoc_insertion_point(class_scope:models.Root.MapStringToStringFieldEntry)
    })
  ,

  'MapStringToMessageFieldEntry' : _reflection.GeneratedProtocolMessageType('MapStringToMessageFieldEntry', (_message.Message,), {
    'DESCRIPTOR' : _ROOT_MAPSTRINGTOMESSAGEFIELDENTRY,
    '__module__' : 'models_pb2'
    # @@protoc_insertion_point(class_scope:models.Root.MapStringToMessageFieldEntry)
    })
  ,

  'Embedded' : _reflection.GeneratedProtocolMessageType('Embedded', (_message.Message,), {
    'DESCRIPTOR' : _ROOT_EMBEDDED,
    '__module__' : 'models_pb2'
    # @@protoc_insertion_point(class_scope:models.Root.Embedded)
    })
  ,

  'ListWrapper' : _reflection.GeneratedProtocolMessageType('ListWrapper', (_message.Message,), {
    'DESCRIPTOR' : _ROOT_LISTWRAPPER,
    '__module__' : 'models_pb2'
    # @@protoc_insertion_point(class_scope:models.Root.ListWrapper)
    })
  ,

  'MapWrapper' : _reflection.GeneratedProtocolMessageType('MapWrapper', (_message.Message,), {

    'DataEntry' : _reflection.GeneratedProtocolMessageType('DataEntry', (_message.Message,), {
      'DESCRIPTOR' : _ROOT_MAPWRAPPER_DATAENTRY,
      '__module__' : 'models_pb2'
      # @@protoc_insertion_point(class_scope:models.Root.MapWrapper.DataEntry)
      })
    ,
    'DESCRIPTOR' : _ROOT_MAPWRAPPER,
    '__module__' : 'models_pb2'
    # @@protoc_insertion_point(class_scope:models.Root.MapWrapper)
    })
  ,

  'InlineEmbedding' : _reflection.GeneratedProtocolMessageType('InlineEmbedding', (_message.Message,), {

    'NestedEmbedding' : _reflection.GeneratedProtocolMessageType('NestedEmbedding', (_message.Message,), {
      'DESCRIPTOR' : _ROOT_INLINEEMBEDDING_NESTEDEMBEDDING,
      '__module__' : 'models_pb2'
      # @@protoc_insertion_point(class_scope:models.Root.InlineEmbedding.NestedEmbedding)
      })
    ,
    'DESCRIPTOR' : _ROOT_INLINEEMBEDDING,
    '__module__' : 'models_pb2'
    # @@protoc_insertion_point(class_scope:models.Root.InlineEmbedding)
    })
  ,
  'DESCRIPTOR' : _ROOT,
  '__module__' : 'models_pb2'
  # @@protoc_insertion_point(class_scope:models.Root)
  })
_sym_db.RegisterMessage(Root)
_sym_db.RegisterMessage(Root.MapStringToStringFieldEntry)
_sym_db.RegisterMessage(Root.MapStringToMessageFieldEntry)
_sym_db.RegisterMessage(Root.Embedded)
_sym_db.RegisterMessage(Root.ListWrapper)
_sym_db.RegisterMessage(Root.MapWrapper)
_sym_db.RegisterMessage(Root.MapWrapper.DataEntry)
_sym_db.RegisterMessage(Root.InlineEmbedding)
_sym_db.RegisterMessage(Root.InlineEmbedding.NestedEmbedding)


_ROOT_MAPSTRINGTOSTRINGFIELDENTRY._options = None
_ROOT_MAPSTRINGTOMESSAGEFIELDENTRY._options = None
_ROOT_MAPWRAPPER_DATAENTRY._options = None
# @@protoc_insertion_point(module_scope)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
try:
    from setuptools import find_packages, setup
except ImportError:
    from distutils.core import setup, find_packages


with open(os.path.join(os.path.dirname(__file__), 'README.rst')) as readme:
    README = readme.read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='django-pb-model',
    version='0.3.2',
    packages=find_packages(),
    include_package_data=True,
    license='MIT License',
    long_description=README,
    url='https://github.com/myyang/django-pb-model',
    description='Protobuf mixin for Django model',
    author='myyang',
    author_email='ymy1019@gmail.com',
    install_requires=[
        'django>=1.11',
        'protobuf>=3.1',
    ],
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
)
